#include "key.h"


#define LED_RED 10
#define LED_GREEN 9

void setup() {
  // 串口初始化
  UartInit();

  // 两个用户LED灯。
  LedInit();

  // 按键初始化，参数是按键处理的回调函数。
  HalKeyInit((void*)handleKeyEvent);

  // 打印一条启动信息
  log("System Running...");
  
  // 两个LED等闪烁一下，指示程序已经运行，同时可以检查LED灯是否有异常。
  LedRedCtrl(true);
  LedGreenCtrl(true);
  delay(50);
  LedRedCtrl(false);
  LedGreenCtrl(false);
  
  // 等待连接的模组正常运行。
  delay(500);
}

// 注意，所有轮训不能阻塞，需要使用状态机。
void loop() {
  // 1-USB串口轮询
  handleSerialLoop();
  // 2-模块串口轮询
  handleSerial1Loop();
  
  // 3-按键轮训
  HalKeyPoll();

  // 4-其他任务轮询
  // code here
}


// 处理两种数据： 1自定义的点灯命令；2蓝牙连接/断开事件
// 1-自定义点灯命令，
//      ON 点亮绿灯
//      OFF 关闭绿灯
// 2-蓝牙连接/断开事件
//      +CONN=x,xxx 表示蓝牙已连接
//      +DISCONN=x,xxx,xxxx 表示蓝牙已断开
void BlueDataParse(char *data, int len)
{
  String msg = String(data);
  // 简单的判断指令头，不做详细的验证。
  if(msg.startsWith("\r\n+CONN=")){
    // 表示已连接，点亮红灯。 
    LedRedCtrl(true);
  }else if(msg.startsWith("\r\n+DISCONN=")){
    // 表示已连接，点亮红灯。 
    LedRedCtrl(false);
  }

  // 对比是否是期望的指令。
  if(msg.equalsIgnoreCase("ON")){
    LedGreenCtrl(true);
  }else if(msg.equalsIgnoreCase("OFF")){
    LedGreenCtrl(false);
  }
}


// 按键回调函数，在这里处理按键任务。
// 关于按键的实现机制请阅读 key.ino 中的代码
void handleKeyEvent(uint8_t keys, uint8_t lastKeys){
  unsigned long time = 0;
  uint8_t ret = 0;

  // 按键S1
  ret = KeyPressed(keys,lastKeys,HAL_KEY_BUTTON_S1, &time);
  if(ret == KEY_PRESSED){
    uint8_t msg[] = "#Key1";
    ModuleWrite(msg, sizeof(msg) / sizeof(msg[0]));
    log("Key1 Pressed.");
  }

  // 按键S2
  ret = KeyPressed(keys, lastKeys, HAL_KEY_BUTTON_S2, &time);
  if(ret == KEY_PRESSED){
    // 通过BLE发送出去。
    uint8_t msg[] = "##Key2";
    ModuleWrite(msg, sizeof(msg) / sizeof(msg[0]));
    log("Key2 Pressed.");
  }
}


void UartInit()
{
  // USB串口
  Serial.begin(115200);
  // 9600波特率不能低于4ms
  Serial.setTimeout(4);
  
  // 模块串口
  Serial1.begin(115200);
  // 9600波特率不能低于4ms
  Serial1.setTimeout(4); 

}

char buf[256] = {0};
char buf1[256] = {0};
// 接收USB的串口数据
void handleSerialLoop(){
  if(Serial.available()){
    memset(buf,0,256);
    int len = Serial.readBytes(buf, 256);
    // 原封不动转发给模组
    Serial1.write(buf, len);
  }
}

// 接收模块的串口数据
void handleSerial1Loop(){
  if(Serial1.available()){
    memset(buf1,0,256);
    int len = Serial1.readBytes(buf1, 256);
    // 原封不动转发给USB
    Serial.write(buf1, len);

    // 解析Ble数据，查看是否是灯孔数据
    BlueDataParse(buf1,len);
  }
}

void LedInit()
{
  // 红灯
  pinMode(LED_RED, OUTPUT);
  digitalWrite(LED_RED, LOW);

  // 绿灯
  pinMode(LED_GREEN, OUTPUT);
  digitalWrite(LED_GREEN, LOW);
}

// true 亮灯; false 关灯
void LedRedCtrl(bool on)
{
  if(on){
    digitalWrite(LED_RED, HIGH);
  }else{
    digitalWrite(LED_RED, LOW);
  }
}

// true 亮灯; false 关灯
void LedGreenCtrl(bool on)
{
  if(on){
    digitalWrite(LED_GREEN, HIGH);
  }else{
    digitalWrite(LED_GREEN, LOW);
  }
}
// 向模块串口发送数据
void ModuleWrite(uint8_t *data, int len)
{
  Serial1.write(data, len);
}

// 通过串口打印log日志。
void log(String msg)
{
  Serial.println("LOG: " + msg);
}
